import Copyright.Copyright;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Implementation of 2D rigid registration algorithm due to B.K.P.Horn:
 *
 * @author Ziv Yaniv (zivy@georgetown.edu)
 */
public class RigidRegistrationAlgorithm {
    public EuclideanTransformation2D apply(Vector fiducials) {
    Enumeration fiducialsE;

            //compute the mean point in both data sets
    Point2D meanFirst = new Point2D();
    Point2D meanSecond = new Point2D(); 
    fiducialsE = fiducials.elements();
    while(fiducialsE.hasMoreElements()) {
        PointPair pair = (PointPair) fiducialsE.nextElement();
        meanFirst.x += pair.first.x;
        meanFirst.y += pair.first.y;
        meanSecond.x += pair.second.x;
        meanSecond.y += pair.second.y;
    }
    int numPoints = fiducials.size();
    meanFirst.x/=numPoints;
    meanFirst.y/=numPoints;
    meanSecond.x/=numPoints;
    meanSecond.y/=numPoints;

                    //center the data around the means
    fiducialsE = fiducials.elements();
    while(fiducialsE.hasMoreElements()) {
        PointPair pair = (PointPair) fiducialsE.nextElement();
        pair.first.x -= meanFirst.x;
        pair.first.y -= meanFirst.y;
        pair.second.x -= meanSecond.x;
        pair.second.y -= meanSecond.y;
    }

    float C,S;
    C = S = 0;
    fiducialsE = fiducials.elements();
    while(fiducialsE.hasMoreElements()) {
        PointPair pair = (PointPair) fiducialsE.nextElement();    
        C += pair.first.x * pair.second.x +
         pair.first.y * pair.second.y;
        S += pair.first.y*pair.second.x -
         pair.first.x*pair.second.y;
    }

    float angle = (float) Math.atan2(S,C);
    float cTheta = (float) Math.cos(angle);
    float sTheta = (float) Math.sin(angle);
    float dx = meanFirst.x - 
               cTheta*meanSecond.x + 
               sTheta*meanSecond.y;
    float dy = meanFirst.y - 
               sTheta*meanSecond.x - 
               cTheta*meanSecond.y;
    return(new EuclideanTransformation2D(dx,dy,angle,false));
    }
}
